package calc.cli;

import static javax.xml.ws.BindingProvider.ENDPOINT_ADDRESS_PROPERTY;

import java.util.Map;

import javax.xml.ws.BindingProvider;

import calc.CalcPortType;
import calc.CalcService;
import calc.DivideByZero;
import pt.ulisboa.tecnico.sdis.ws.uddi.UDDINaming;

/**
 * Calculator client.
 *
 * This class wraps the Port generated by wsimport and 
 * adds easier endpoint address configuration.
 * 
 * It implements the PortType interface to provide access to
 * exactly the same operations as the Port generated by wsimport.
 */
public class CalcClient implements CalcPortType {

	/** WS service */
	CalcService service = null;

	/** WS port (port type is the interface, port is the implementation) */
	CalcPortType port = null;

	/** UDDI server URL */
	private String uddiURL = null;

	/** WS name */
	private String wsName = null;

	/** WS endpoint address */
	private String wsURL = null; // default value is defined inside WSDL

	public String getWsURL() {
		return wsURL;
	}

	/** output option **/
	private boolean verbose = false;

	public boolean isVerbose() {
		return verbose;
	}

	public void setVerbose(boolean verbose) {
		this.verbose = verbose;
	}

	/** constructor with provided web service URL */
	public CalcClient(String wsURL) throws CalcClientException {
		this.wsURL = wsURL;
		createStub();
	}

	/** constructor with provided UDDI location and name */
	public CalcClient(String uddiURL, String wsName) throws CalcClientException {
		this.uddiURL = uddiURL;
		this.wsName = wsName;
		uddiLookup();
		createStub();
	}

	/** UDDI lookup */
	private void uddiLookup() throws CalcClientException {
		try {
			if (verbose)
				System.out.printf("Contacting UDDI at %s%n", uddiURL);
			UDDINaming uddiNaming = new UDDINaming(uddiURL);

			if (verbose)
				System.out.printf("Looking for '%s'%n", wsName);
			wsURL = uddiNaming.lookup(wsName);

		} catch (Exception e) {
			String msg = String.format("Client failed lookup on UDDI at %s!", uddiURL);
			throw new CalcClientException(msg, e);
		}

		if (wsURL == null) {
			String msg = String.format("Service with name %s not found on UDDI at %s", wsName, uddiURL);
			throw new CalcClientException(msg);
		}
	}

	/** Stub creation and configuration */
	private void createStub() {
		if (verbose)
			System.out.println("Creating stub ...");
		service = new CalcService();
		port = service.getCalcPort();

		if (wsURL != null) {
			if (verbose)
				System.out.println("Setting endpoint address ...");
			BindingProvider bindingProvider = (BindingProvider) port;
			Map<String, Object> requestContext = bindingProvider.getRequestContext();
			requestContext.put(ENDPOINT_ADDRESS_PROPERTY, wsURL);
		}
	}

	// CalcPortType

	public int sum(int a, int b) {
		return port.sum(a, b);
	}

	public int sub(int a, int b) {
		return port.sub(a, b);
	}

	public int mult(int a, int b) {
		return port.mult(a, b);
	}

	public int intdiv(int a, int b) throws DivideByZero {
		return port.intdiv(a, b);
	}

}
